@@ -28,12 +28,15 @@ import datetime
 import gdata.calendar.service
 import googlecl
 import googlecl.service
+import logging
+import time
+import urllib
 from googlecl.calendar import SECTION_HEADER
 
 
+LOG = logging.getLogger(googlecl.calendar.LOGGER_NAME)
 USER_BATCH_URL_FORMAT = \
                gdata.calendar.service.DEFAULT_BATCH_URL.replace('default', '%s')
-QUERY_DATE_FORMAT = '%Y-%m-%dT%H:%S:%M'
 
 class CalendarError(googlecl.service.Error):
   """Base error for Calendar errors."""
@@ -44,6 +47,27 @@ class EventsNotFound(CalendarError):
   pass
 
 
+class Calendar():
+  
+  """Wrapper class for some calendar entry data."""
+
+  def __init__(self, cal_entry=None, user=None, name=None):
+    """Parse a CalendarEntry into "user" and human-readable names,
+       or take them directly."""
+    if cal_entry:
+      # Non-primary calendar feeds look like this:
+      # http:blah/.../feeds/JUNK%40group.calendar.google.com/private/full
+      # So grab the part after /feeds/ and unquote it.
+      self.user = urllib.unquote(cal_entry.content.src.split('/')[-3])
+      self.name = cal_entry.title.text
+    else:
+      self.user = user
+      self.name = name
+
+  def __str__(self):
+    return self.name
+
+
 class CalendarServiceCL(gdata.calendar.service.CalendarService,
                         googlecl.service.BaseServiceCL):
 
@@ -54,21 +78,10 @@ class CalendarServiceCL(gdata.calendar.service.CalendarService,
 
   """
 
-  def __init__(self, regex=False, tags_prompt=False, delete_prompt=True):
-    """Constructor.
-    
-    Keyword arguments:
-      regex: Indicates if regular expressions should be used for matching
-             strings, such as event titles. (Default False)
-      tags_prompt: Indicates if while inserting events, instance should prompt
-                   for tags for each photo. (Default False)
-      delete_prompt: Indicates if instance should prompt user before
-                     deleting a calendar or event. (Default True)
-              
-    """
+  def __init__(self):
+    """Constructor."""
     gdata.calendar.service.CalendarService.__init__(self)
-    googlecl.service.BaseServiceCL._set_params(self, regex,
-                                               tags_prompt, delete_prompt)
+    self._set_params(SECTION_HEADER)
 
   def _batch_delete_recur(self, event, cal_user,
                           start_date=None, end_date=None):
@@ -85,6 +98,34 @@ class CalendarServiceCL(gdata.calendar.service.CalendarService,
     map(request_feed.AddDelete, [None], delete_events, [None])
     self.ExecuteBatch(request_feed, USER_BATCH_URL_FORMAT % cal_user)
 
+  def add_reminders(self, calendar_user, events, minutes):
+    """Add default reminders to events.
+
+    Keyword arguments:
+      calendar_user: "User" of the calendar.
+      events: List of events to add reminder to.
+      minutes: Number of minutes before each event to send reminder.
+
+    Returns:
+      List of events with batch results.
+
+    """
+    request_feed = gdata.calendar.CalendarEventFeed()
+    for event in events:
+      if event.when:
+        for a_when in event.when:
+          a_when.reminder.append(gdata.calendar.Reminder(minutes=minutes))
+      else:
+        LOG.debug('No "when" data for event!')
+        event.when.append(gdata.calendar.When())
+        event.when[0].reminder.append(gdata.calendar.Reminder(minutes=minutes))
+      request_feed.AddUpdate(entry=event)
+    response_feed = self.ExecuteBatch(request_feed,
+                                      USER_BATCH_URL_FORMAT % calendar_user)
+    return response_feed.entry
+
+  AddReminders = add_reminders
+
   def delete_events(self, events, date, calendar_user):
     """Delete events from a calendar.
     
@@ -105,7 +146,7 @@ class CalendarServiceCL(gdata.calendar.service.CalendarService,
     delete_default = googlecl.CONFIG.getboolean('GENERAL', 'delete_by_default')
     self.Delete(single_events, 'event', delete_default)
     
-    start_date, end_date, start_date_utc, end_date_utc = get_start_and_end(date)
+    date = googlecl.calendar.Date(date)
     # option_list is a list of tuples, (prompt_string, deletion_instruction)
     # prompt_string gets displayed to the user,
     # deletion_instruction is a special value that will let the program know
@@ -116,14 +157,12 @@ class CalendarServiceCL(gdata.calendar.service.CalendarService,
     #     'ON' -- delete events on the single date given.
     #     'ONAFTER' -- delete events on and after the date given.
     option_list = [('All events in this series', 'ALL')]
-    if start_date and end_date:
-      option_list.append(('Instances between ' + start_date + ' and ' +
-                          end_date, 'TWIXT'))
-    elif start_date or end_date:
-      if (start_date and not start_date_utc) or (end_date and not end_date_utc):
-        raise CalendarError('UTC date not set when TZ date is!')
-      delete_date = (start_date or end_date)
-      delete_date_utc = (start_date_utc or end_date_utc)
+    if date.start and date.end:
+      option_list.append(('Instances between ' + date.start + ' and ' +
+                          date.end, 'TWIXT'))
+    elif date.start or date.end:
+      delete_date = (date.start or date.end)
+      delete_date_utc = (date.utc_start or date.utc_end)
       option_list.append(('Instances on ' + delete_date,
                           'ON'))
       option_list.append(('All events on and after ' + delete_date,
@@ -143,13 +182,13 @@ class CalendarServiceCL(gdata.calendar.service.CalendarService,
           gdata.service.GDataService.Delete(self, event.GetEditLink().href)
         elif option[1] == 'TWIXT':
           self._batch_delete_recur(event, calendar_user,
-                                   start_date=start_date_utc,
-                                   end_date=end_date_utc)
+                                   start_date=date.utc_start,
+                                   end_date=date.utc_end)
         elif option[1] == 'ON':
-          start_date_utc, end_date_utc = _tomorrowize(delete_date_utc)
+          new_date = googlecl.calendar.Date(delete_date_utc)
           self._batch_delete_recur(event, calendar_user,
-                                   start_date=start_date_utc,
-                                   end_date=end_date_utc)
+                                   start_date=new_date.utc_start,
+                                   end_date=new_date.utc_end)
         elif option[1] == 'ONAFTER':
           self._batch_delete_recur(event, calendar_user,
                                    start_date=delete_date_utc)
@@ -179,55 +218,50 @@ class CalendarServiceCL(gdata.calendar.service.CalendarService,
       event = gdata.calendar.CalendarEventEntry()
       event.content = atom.Content(text=event_str)
       event.quick_add = gdata.calendar.QuickAdd(value='true')
-      request_feed.AddInsert(event, 'insert-request' + str(i))
+      request_feed.AddInsert(event, 'insert-' + event_str[0:5] + str(i))
     response_feed = self.ExecuteBatch(request_feed,
                                       USER_BATCH_URL_FORMAT % calendar_user)
     return response_feed.entry
 
   QuickAddEvent = quick_add_event
 
-  def get_calendar_user(self, cal_name=None):
-    """Get "user" name for one calendar.
+  def get_calendar_user_list(self, cal_name=None):
+    """Get "user" name and human-readable name for one or more calendars.
     
     The "user" for a calendar is an awful misnomer for the ID for the calendar.
     To get events for a calendar, you can form a query with
-      user = self.get_calendar_user('my calendar name')
-      if user:
-        query = gdata.calendar.CalendarEventQuery(user=user)
+      cal_list = self.get_calendar_user_list('my calendar name')
+      if cal_list:
+        query = gdata.calendar.CalendarEventQuery(user=cal_list[0].user)
     
     Keyword arguments:
       cal_name: Name of the calendar to match. Default None to return the 
-                uri of the default / main calendar.
+                an instance representing only the default / main calendar.
       
     Returns:
-      Single CalendarEntry, or None of there were no matches for cal_name.
+      A list of Calendar instances, or None of there were no matches
+      for cal_name.
     
     """
-    import urllib
-    if not cal_name or cal_name == 'default':
-      return 'default'
+    if not cal_name:
+      return [Calendar(user='default', name=self.email)]
     else:
-      cal = self.GetSingleEntry('/calendar/feeds/default/allcalendars/full',
-                                 cal_name,
-                            converter=gdata.calendar.CalendarListFeedFromString)
-      if cal:
-        # Non-primary calendar feeds look like this:
-        # http:blah/blah/.../feeds/JUNK%40group.calendar.google.com/private/full
-        # So grab the part after /feeds/ and unquote it.
-        return urllib.unquote(cal.content.src.split('/')[-3])
-      else:
-        return None
+      cal_list = self.GetEntries('/calendar/feeds/default/allcalendars/full',
+                                 title=cal_name,
+                          converter=gdata.calendar.CalendarListFeedFromString)
+      if cal_list:
+        return [Calendar(cal) for cal in cal_list]
+    return None
 
-  GetCalendarUser = get_calendar_user
+  GetCalendarUserList = get_calendar_user_list
 
   def get_events(self, calendar_user, start_date=None, end_date=None,
-                 title=None, query=None, max_results=100,
-                 expand_recurrence=True):
+                 title=None, query=None, expand_recurrence=True):
     """Get events.
     
     Keyword arguments:
       calendar_user: "user" of the calendar to get events for.
-                     See get_calendar_user.
+                     See get_calendar_user_list.
       start_date: Start date of the event(s). Must follow the RFC 3339
                   timestamp format and be in UTC. Default None.
       end_date: End date of the event(s). Must follow the RFC 3339 timestamp
@@ -236,7 +270,6 @@ class CalendarServiceCL(gdata.calendar.service.CalendarService,
              Default None for any title.
       query: Query string (not encoded) for doing full-text searches on event
              titles and content.
-      max_results: Maximum number of events to get. Default 100.
       expand_recurrence: If true, expand recurring events per the 'singleevents'
                          query parameter. Otherwise, don't.
     
@@ -254,7 +287,6 @@ class CalendarServiceCL(gdata.calendar.service.CalendarService,
       query.singleevents = 'true'
     query.orderby = 'starttime'
     query.sortorder = 'ascend'
-    query.max_results = max_results
     return self.GetEntries(query.ToUri(), title,
                            converter=gdata.calendar.CalendarEventFeedFromString)
 
@@ -270,6 +302,82 @@ class CalendarServiceCL(gdata.calendar.service.CalendarService,
 SERVICE_CLASS = CalendarServiceCL
 
 
+class CalendarEntryToStringWrapper(googlecl.service.BaseEntryToStringWrapper):
+  @property
+  def when(self):
+    """When event takes place."""
+    start_time_data, end_time_data, freq = get_datetimes(self.entry)
+    print_format = googlecl.get_config_option(SECTION_HEADER,
+                                              'date_print_format')
+    start_time = time.strftime(print_format, start_time_data)
+    end_time = time.strftime(print_format, end_time_data)
+    value = start_time + ' - ' + end_time
+    if freq:
+      if freq.has_key('BYDAY'):
+        value += ' (' + freq['BYDAY'].lower() + ')'
+      else:
+        value += ' (' + freq['FREQ'].lower() + ')'
+    return value
+
+  @property
+  def where(self):
+    """Where event takes place"""
+    return self._join(self.entry.where, text_attribute='value_string')
+
+
+def convert_reminder_string(reminder):
+  """Convert reminder string to minutes integer.
+
+  Keyword arguments:
+    reminder: String representation of time,
+              e.g. '10' for 10 minutes,
+                   '1d' for one day,
+                   '3h' for three hours, etc.
+  Returns:
+    Integer of reminder converted to minutes.
+
+  Raises:
+    ValueError if conversion failed.
+
+  """
+  if not reminder:
+    return None
+  unit = reminder.lower()[-1]
+  value = reminder[:-1]
+  if unit == 's':
+    return int(value) / 60
+  elif unit == 'm':
+    return int(value)
+  elif unit == 'h':
+    return int(value) * 60
+  elif unit == 'd':
+    return int(value) * 60 * 24
+  elif unit == 'w':
+    return int(value) * 60 * 24 * 7
+  else:
+    return int(reminder)
+
+
+def get_date_today(include_hour=False, as_range=False):
+  """Get today's date, as if entered by the user.
+
+  Keyword arguments:
+    include_hour: Include the hour in the return string. Default False.
+
+  Returns:
+    String representation of today's date
+
+  """
+  date_data = datetime.datetime.today()
+  if include_hour:
+    date_str = date_data.strftime(googlecl.calendar.QUERY_DATE_FORMAT)
+  else:
+    date_str = date_data.strftime(googlecl.service.DATE_FORMAT)
+  if as_range:
+    date_str += ','
+  return date_str
+
+
 def get_datetimes(cal_entry):
   """Get datetime objects for the start and end of the event specified by a
   calendar entry.
@@ -285,16 +393,16 @@ def get_datetimes(cal_entry):
            event does not repeat (does not have a gd:recurrence element)).
   
   """
-  import time
   if cal_entry.recurrence:
     return parse_recurrence(cal_entry.recurrence.text)
   else:
     freq = None
     when = cal_entry.when[0]
     try:
-      start_time_data = time.strptime(when.start_time[:-10],
+      # Trim the string data from "when" to only include down to seconds
+      start_time_data = time.strptime(when.start_time[:19],
                                       '%Y-%m-%dT%H:%M:%S')
-      end_time_data = time.strptime(when.end_time[:-10],
+      end_time_data = time.strptime(when.end_time[:19],
                                     '%Y-%m-%dT%H:%M:%S')
     except ValueError:
       # Try to handle date format for all-day events
@@ -303,58 +411,6 @@ def get_datetimes(cal_entry):
   return (start_time_data, end_time_data, freq)
 
 
-def get_start_and_end(date):
-  """Split a string representation of a date or range of dates.
-  
-  Ranges should be designated via a comma. For example, '2010-06-01,2010-06-20'
-  will set return ('2010-06-01', '2010-06-20', ...)
-  
-  Returns:
-    Tuple of (start, end, utc_start, utc_end) where
-       start is either the starting date or None,
-       end is either the ending date or None,
-       utc_start is the starting date shifted into UTC,
-       utc_end is the ending date shifted into UTC.
-  
-  """
-  if date and date != ',':
-    # Partition won't choke on date == '2010-06-05', split will.
-    start, is_range, end = date.partition(',')
-  else:
-    # If no date is given, set a start of today.
-    start = datetime.datetime.today().strftime(googlecl.service.DATE_FORMAT)
-    is_range = None
-    end = None
-  utc_timedelta = get_utc_timedelta()
-  # Even though the "when" elements of events will be properly shifted into
-  # the user's timezone, all queries are interpreted as UTC (GMT) time.
-  if start:
-    start_time = datetime.datetime.strptime(start, googlecl.service.DATE_FORMAT)
-    utc_start = (start_time + (utc_timedelta)).strftime(QUERY_DATE_FORMAT)
-  else:
-    utc_start = None
-  if not is_range:
-    dates = _tomorrowize(start)
-    # _tomorrowize() returns a full timestamp with hour data, so trim
-    # down to same format that user enters dates in.
-    end = dates[1][:10]
-  if end:
-    end_time = datetime.datetime.strptime(end, googlecl.service.DATE_FORMAT)
-    utc_end = (end_time + (utc_timedelta)).strftime(QUERY_DATE_FORMAT)
-  else:
-    utc_end = None
-  return (start, end, utc_start, utc_end)
-
-
-def get_utc_timedelta():
-  """Return the UTC offset as a timedelta."""
-  import time
-  if time.daylight != 0:
-    return datetime.timedelta(hours=time.altzone/3600)
-  else:
-    return datetime.timedelta(hours=time.timezone/3600)
-
-
 def parse_recurrence(time_string):
   """Parse recurrence data found in event entry.
   
@@ -368,7 +424,6 @@ def parse_recurrence(time_string):
     values. (http://www.ietf.org/rfc/rfc2445.txt, section 4.3.10)
   
   """
-  import time
   # Google calendars uses a pretty limited section of RFC 2445, and I'm
   # abusing that here. This will probably break if Google ever changes how
   # they handle recurrence, or how the recurrence string is built.
@@ -388,31 +443,30 @@ def parse_recurrence(time_string):
   return (start_time, end_time, freq)
 
 
-def _tomorrowize(date=None):
-  """Return a date range from given date until tomorrow.
-  
-  Keyword arguments:
-    date: String of date to start at, following RFC 3339 timestamp format,
-          but does not have to include data past 'YYYY-MM-DD'.
-          Must be in UTC. Default None, for today.
-    
-  Returns:
-    (start_date, end_date) where both dates are strings representing UTC time
-    in the RFC 3339 format. end_date is exactly one day after start_date.
-    
-  """
-  if not date:
-    date_data = datetime.datetime.today()
-  else:
-    try:
-      date_data = datetime.datetime.strptime(date, QUERY_DATE_FORMAT)
-    except ValueError:
-      # If there is no hour data (i.e. the DATE_FORMAT succeeds), add it.
-      date_data = datetime.datetime.strptime(date, googlecl.service.DATE_FORMAT)
-      date_data += get_utc_timedelta()
-  tomorrow_data = date_data + datetime.timedelta(days=1)
-  return (date_data.strftime(QUERY_DATE_FORMAT),
-         tomorrow_data.strftime(QUERY_DATE_FORMAT))
+def _list(client, options, args, date):
+  cal_user_list = client.get_calendar_user_list(options.cal)
+  if not cal_user_list:
+    LOG.error('No calendar matches "' + options.cal + '"')
+    return
+  for cal in cal_user_list:
+    print ''
+    print '[' + str(cal) + ']'
+    entries = client.get_events(cal.user,
+                                start_date=date.utc_start,
+                                end_date=date.utc_end,
+                                title=options.title,
+                                query=options.query)
+
+    if args:
+      style_list = args[0].split(',')
+    else:
+      style_list = googlecl.get_config_option(SECTION_HEADER,
+                                              'list_style').split(',')
+    for entry in entries:
+      print googlecl.service.compile_entry_string(
+                                            CalendarEntryToStringWrapper(entry),
+                                            style_list,
+                                            delimiter=options.delimiter)
 
 
 #===============================================================================
@@ -425,69 +479,64 @@ def _tomorrowize(date=None):
 #        required
 #===============================================================================
 def _run_list(client, options, args):
-  cal_user = client.get_calendar_user(options.cal)
-  if not cal_user:
-    print 'No calendar matches "' + options.cal + '"'
-    return
-  dates = get_start_and_end(options.date)
-  entries = client.get_events(cal_user,
-                              start_date=dates[2],
-                              end_date=dates[3],
-                              title=options.title,
-                              query=options.query)
-  if args:
-    style_list = args[0].split(',')
+  # If no other search parameters are mentioned, set date to be
+  # today. (Prevent user from retrieving all events ever)
+  if not (options.title or options.query or options.date):
+    date = googlecl.calendar.Date(get_date_today(include_hour=True,
+                                                 as_range=True))
   else:
-    style_list = googlecl.get_config_option(SECTION_HEADER,
-                                            'list_style').split(',')
-  for entry in entries:
-    print googlecl.service.entry_to_string(entry, style_list,
-                                           delimiter=options.delimiter)
+    date = googlecl.calendar.Date(options.date)
+  _list(client, options, args, date)
 
 
 def _run_list_today(client, options, args):
-  cal_user = client.get_calendar_user(options.cal)
-  if not cal_user:
-    print 'No calendar matches "' + options.cal + '"'
-    return
-  start_date, end_date = _tomorrowize()
-  entries = client.get_events(cal_user,
-                              start_date=start_date,
-                              end_date=end_date,
-                              title=options.title,
-                              query=options.query)
-  if args:
-    style_list = args[0].split(',')
-  else:
-    style_list = googlecl.get_config_option(SECTION_HEADER,
-                                            'list_style').split(',')
-  for entry in entries:
-    print googlecl.service.entry_to_string(entry, style_list,
-                                           delimiter=options.delimiter)
+  date = googlecl.calendar.Date(get_date_today(include_hour=False))
+  _list(client, options, args, date)
 
 
 def _run_add(client, options, args):
-  cal_user = client.get_calendar_user(options.cal)
-  if not cal_user:
-    print 'No calendar matches "' + options.cal + '"'
+  cal_user_list = client.get_calendar_user_list(options.cal)
+  if not cal_user_list:
+    LOG.error('No calendar matches "' + options.cal + '"')
     return
-  client.quick_add_event(args, cal_user)
+  minutes = convert_reminder_string(options.reminder)
+  for cal in cal_user_list:
+    results = client.quick_add_event(args, cal.user)
+    if LOG.isEnabledFor(logging.DEBUG):
+      for entry in results:
+        LOG.debug('ID: %s, status: %s, reason: %s',
+                  entry.batch_id.text,
+                  entry.batch_status.code,
+                  entry.batch_status.reason)
+      
+    if minutes:
+      new_results = client.add_reminders(cal.user, results, minutes)
+      if LOG.isEnabledFor(logging.DEBUG):
+        for entry in new_results:
+          LOG.debug('ID: %s, status: %s, reason: %s',
+                    entry.batch_id.text,
+                    entry.batch_status.code,
+                    entry.batch_status.reason)
 
 
 def _run_delete(client, options, args):
-  cal_user = client.get_calendar_user(options.cal)
-  if not cal_user:
-    print 'No calendar matches "' + options.cal + '"'
+  cal_user_list = client.get_calendar_user_list(options.cal)
+  if not cal_user_list:
+    LOG.error('No calendar matches "' + options.cal + '"')
     return
-  dates = get_start_and_end(options.date)
-  events = client.get_events(cal_user, start_date=dates[2],
-                             end_date=dates[3],
-                             title=options.title, query=options.query,
-                             expand_recurrence=False)
-  try:
-    client.delete_events(events, options.date, cal_user)
-  except EventsNotFound:
-    print 'No events found that match your options!'
+  date = googlecl.calendar.Date(options.date)
+  for cal in cal_user_list:
+    LOG.info('For calendar ' + str(cal))
+    events = client.get_events(cal.user,
+                               start_date=date.utc_start,
+                               end_date=date.utc_end,
+                               title=options.title,
+                               query=options.query,
+                               expand_recurrence=False)
+    try:
+      client.delete_events(events, options.date, cal.user)
+    except EventsNotFound:
+      LOG.warning('No events found that match your options!')
 
 
 TASKS = {'list': googlecl.service.Task('List events on a calendar',
