@@ -11,7 +11,8 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+import googlecl
 
-
-SECTION_HEADER = 'DOCS'
-
+service_name = __name__.split('.')[-1]
+LOGGER_NAME = googlecl.LOGGER_NAME + '.' + service_name
+SECTION_HEADER = service_name.upper()
